@@ -32,7 +32,7 @@ extern "C" {
  * 0.2.6-1 => 0x00020601
  * 4.5.2-0 => 0x04050200
  */
-#define EXTRACTOR_VERSION 0x00051300
+#define EXTRACTOR_VERSION 0x00051600
 
 #include <stdio.h>
 
@@ -168,6 +168,24 @@ typedef enum {
   EXTRACTOR_INFORMATION = 112,
   EXTRACTOR_FULL_NAME = 113,
   EXTRACTOR_CHAPTER = 114,
+  EXTRACTOR_YEAR = 115,
+
+  EXTRACTOR_LINK = 116,
+  EXTRACTOR_MUSIC_CD_IDENTIFIER = 117,
+  EXTRACTOR_PLAY_COUNTER = 118,
+  EXTRACTOR_POPULARITY_METER = 119,
+  EXTRACTOR_CONTENT_TYPE = 120,
+  EXTRACTOR_ENCODED_BY = 121,
+  EXTRACTOR_TIME = 122,
+  EXTRACTOR_MUSICIAN_CREDITS_LIST = 123,
+  EXTRACTOR_MOOD = 124,
+
+  EXTRACTOR_FORMAT_VERSION = 125,
+  EXTRACTOR_TELEVISION_SYSTEM = 126,
+  EXTRACTOR_SONG_COUNT = 127,
+  EXTRACTOR_STARTING_SONG = 128,
+  EXTRACTOR_HARDWARE_DEPENDENCY = 129,
+
 } EXTRACTOR_KeywordType;
 
 /**
