@@ -80,20 +80,33 @@ typedef struct {
 
 
 /* mimetype = application/zip */
-struct EXTRACTOR_Keywords * libextractor_zip_extract(char * filename,
-                                                     char * data,
-                                                     size_t size,
-                                                     struct EXTRACTOR_Keywords * prev) {
+struct EXTRACTOR_Keywords * 
+libextractor_zip_extract(const char * filename,
+			 const unsigned char * data,
+			 size_t size,
+			 struct EXTRACTOR_Keywords * prev) {
   void * tmp;
   zip_entry * info;
   zip_entry * start;
   char *filecomment = NULL;
-  unsigned char * pos;
+  const unsigned char * pos;
   unsigned int offset, stop;
   unsigned int name_length, extra_length, comment_length;
   unsigned int filecomment_length;
   unsigned int entry_total, entry_count; 
   EXTRACTOR_KeywordList * keyword;
+  const char * mimetype;
+
+  mimetype = EXTRACTOR_extractLast(EXTRACTOR_MIMETYPE, 
+				   prev);
+  if (NULL != mimetype) {
+    if ( (0 != strcmp(mimetype, "application/x-zip")) &&
+	 (0 != strcmp(mimetype, "application/zip")) ) {
+      /* we think we already know what's in here,
+	 and it is not a zip */
+      return prev;
+    }
+  }
 
   /* I think the smallest zipfile you can have is about 120 bytes */
   if ( (NULL==data) || (size < 100) )
